import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart';
import 'package:flutterskel/datasource/DataSource.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart';
import 'package:flutterskel/resources/dimensions/DimensionResources.dart';
import 'package:flutterskel/resources/fonts/FontResources.dart';
import 'package:flutterskel/resources/icons/IconResources.dart';
import 'package:flutterskel/resources/images/ImageResources.dart';
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart';
import 'package:flutterskel/resources/themes/ThemeResources.dart';
import 'package:flutterskel/routing/page/PageRouter.dart';
import 'package:flutterskel/routing/url/UrlRouter.dart';
import 'package:flutterskel/userinformation/UserInformationRepository.dart';
import 'package:provider/provider.dart';

/// abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatefulWidgetState<
                  StateWidget extends StatefulWidget,
                  // Data
                  Data extends DataSource,
                  // Routing
                  URouter extends UrlRouter,
                  PRouter extends PageRouter,
                  // Configuration
                  ConfigRepo extends ConfigurationRepository,
                  // User Information
                  UserRepo extends UserInformationRepository,
                  // Strings
                  StringRepo extends StringsRepository,
                  // Themes (Fonts/Dimensions/Colors)
                  ThemeRepo extends ThemeRepository,
                  Theme extends ThemeResources,
                  Colors extends ColorResources,
                  Dimensions extends DimensionResources,
                  Fonts extends FontResources,
                  // Images and Icons
                  Images extends ImageResources,
                  Icons extends IconResources> extends State<StateWidget> {

  Data get dataSource => Provider.of<Data>(context);

  URouter get urlRouter => Provider.of<URouter>(context);

  PRouter get pageRouter => Provider.of<PRouter>(context);

  Images get images => Provider.of<Images>(context);

  ConfigRepo get configuration => Provider.of<ConfigRepo>(context);

  UserRepo get userInformation => Provider.of<UserRepo>(context);

  StringRepo get strings => Provider.of<StringRepo>(context);

  ThemeRepo get themes => Provider.of<ThemeRepo>(context);

  Theme get theme => Provider.of<Theme>(context);

  Colors get colors => Provider.of<Colors>(context);

  Dimensions get dimensions => Provider.of<Dimensions>(context);

  Fonts get fonts => Provider.of<Fonts>(context);

  Icons get icons => Provider.of<Icons>(context);
}
